'use strict'

const async = require('async')

const dwfAsync = require('./dwfAsync.js')
const logger = require('./logger.js')

// deploy dwf content
exports.deploy = function (options, targetSrvMin, content, cb) {
  const client = dwfAsync(targetSrvMin.credentials, options)
  let deployContent
  const tasks = []
  const serverTopDirs = content.serverTopDirs() || []

  logger.trace(serverTopDirs)

  logger.logTimerInit('preprocessing-files', 'Preprocessing files...')
  try {
    deployContent = content.deployContent()
  } catch (err) {
    cb(err)
    return
  }
  logger.logTimerDelta('preprocessing-files', 'Preprocessing files... ok')

  tasks.push(logger.logfnTimerInit('deploying-files', 'Deploying to the container "%s"...', targetSrvMin.name))
  tasks.push(client.designToRuntime(deployContent))
  tasks.push(logger.logfnTimerDelta('deploying-files', 'Deploying to the container "%s"... ok', targetSrvMin.name))

  async.series(tasks, function (err, results) {
    if (err) {
      logger.fnDeployServiceError(targetSrvMin.name)
    } else {
      logger.fnDeployServiceSuccess(targetSrvMin.name)
    }

    cb(err, {
      task: 'deploy',
      results: results
    })
  })
}
