'use strict'

const Result = require('../../result.js').Result
const coDwfAbstract = require('../../abstract.js')

const coAsyncLib = require('async')
const coAbstractLib = require('./abstract.js')
const coValidateLib = require('../../validate.js')
const coRepositoryLib = require('../../repository.js')

const cfPostBatch = function (isExt, iaoPostBatch, iaoResultItem, ioProviderExt, ioCredential) {
  const laoObjectInstanceToDelete = []
  const laoObjectInstanceToPost = []
  const laoResultItem = iaoResultItem

  iaoPostBatch.forEach(function (ioDesignObject) {
    if (ioDesignObject[coAbstractLib.csKey_AOI_Action] === coAbstractLib.Methods.POST) {
      const loRuntimeObject = {
        NAMESPACE: ioDesignObject[coAbstractLib.csKey_AOI_csNamespace],
        ID: ioDesignObject[coAbstractLib.csKey_AOI_DLM_Profile_Id],
        JSON: JSON.stringify(ioDesignObject.data)
      }
      laoObjectInstanceToPost.push(loRuntimeObject)
    }
  })
  laoResultItem.forEach(function (ioResultItem) {
    if (ioResultItem[coAbstractLib.csKey_AOI_Action] === coAbstractLib.Methods.DELETE) {
      const loObjectInstance = {
        NAMESPACE: ioResultItem[coAbstractLib.csKey_AOI_csNamespace],
        ID: ioResultItem[coAbstractLib.csKey_AOI_DLM_Profile_Id]
      }
      laoObjectInstanceToDelete.push(loObjectInstance)
    }
  })

  const lafPost = coAbstractLib.fpreparePost(isExt, laoObjectInstanceToPost, ioProviderExt, ioCredential, coAbstractLib.Methods.POST)
  const lafDelete = coAbstractLib.fpreparePost(isExt, laoObjectInstanceToDelete, ioProviderExt, ioCredential, coAbstractLib.Methods.DELETE)
  let lafTask = []
  if (lafPost.length > 0) {
    lafTask = lafTask.concat(lafPost)
  }
  if (lafDelete.length > 0) {
    lafTask = lafTask.concat(lafDelete)
  }

  return lafTask
}

const cfPostRepo = function (isExt, iaoResultItem, ioProviderExt, ioObjectProperties, ioCredential, ioOption, ioProviders, ioObjectType) {
  const loRepository = new coRepositoryLib.Repository(ioCredential, ioOption)
  const loObjectProperties = ioObjectProperties
  const lafTaskWaterfall = loRepository.get_afTask2Update(isExt, iaoResultItem, ioProviders, ioObjectType, loObjectProperties, ioProviders, ioObjectType)
  const lafTask = []
  lafTask.push(function (ifCb) {
    coAsyncLib.waterfall(lafTaskWaterfall, function (ioError, ioResult) {
      ifCb(ioError, ioResult)
    })
  })
  return lafTask
}

const coType = {}
coType[coAbstractLib.csExtention] = '.dwfdlmprofile'
coType[coAbstractLib.cfPrepare] = undefined
coType[coAbstractLib.cfIsDeployable] = function () {
  const fCreate = coType[coAbstractLib.cfCreateTasks]
  if (fCreate) {
    return true
  } else {
    return false
  }
}
coType[coAbstractLib.cfCommonObjectProperties] = function () {
  return {
    NAMESPACE: coAbstractLib.csKey_AOI_csNamespace,
    ID: coAbstractLib.csKey_AOI_DLM_Profile_Id,
    TYPE_ID: coAbstractLib.csKey_AOI_TYPE_ID,
    DESIGN_TIME_FILE_PATH: coAbstractLib.csKey_AOI_sFile,
    HASH_UTF8_SHA256_HEX: coAbstractLib.csKey_AOI_sHashUtf8Sha256Hex
  }
}
coType[coAbstractLib.cfCreateTasks] = function (iaoPostBatch, ioResult, ioCredential, ioOption, ioProviders, ioObjectType) {
  coValidateLib(iaoPostBatch.length, 'number')
  coValidateLib(ioResult, 'object')
  coValidateLib(ioCredential, 'object')
  coValidateLib(ioOption, 'object')
  const loTempResultObj = new Result()
  const loResult = ioResult
  const lsExt = coType[coAbstractLib.csExtention]
  const laoPostBatch = iaoPostBatch
  let loValue4afTask = []

  const loProviderExt = coAbstractLib.cfGetProvider(ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1], lsExt)
  const lafTaskPostBatch = cfPostBatch(lsExt, laoPostBatch, loResult, loProviderExt, ioCredential)
  loValue4afTask = loValue4afTask.concat(lafTaskPostBatch)

  const loObjectProperties = coType[coAbstractLib.cfCommonObjectProperties]()
  const lafTaskPostRepo = cfPostRepo(lsExt, loResult, loProviderExt, loObjectProperties, ioCredential, ioOption, ioProviders, ioObjectType)
  loValue4afTask = loValue4afTask.concat(lafTaskPostRepo)

  loTempResultObj.addValue(lsExt, loValue4afTask)

  return loTempResultObj
}

module.exports = coType
