'use strict'

const logger = require('./logger.js')

const coAsyncLib = require('async')
const coContainerLib = require('./dwf/container.js')
const coRepositoryLib = require('./dwf/repository.js')
const coAbstractLib = require('./dwf/object/type/abstract.js')

module.exports = function (ioCredentials, ioOptions) {
  const credentials = ioCredentials
  const options = ioOptions

  function designToRuntime (iaaFiles) {
    return function (cb) {
      try {
        let loContainer
        const loRepository = new coRepositoryLib.Repository(credentials, options)
        /* it contains 3 results from 3 GET calls
        *  getResults[0] --> repo/objects
        *  getResults[1] --> repo/providers
        *  getResults[2] --> repo/ObjectTypes
        */
        let liIndex = 0
        const csRepoLinks = [coAbstractLib.TYPES.OBJECTS, coAbstractLib.TYPES.PROVIDERS, coAbstractLib.TYPES.OBJECT_TYPES]
        const getResults = {}
        // It's a GET call on  URL/repo/objects to get all .dwftaskchain file
        const lfcb = function (res, cb) {
          if (res && res.body && res.body.Result) {
            const csString = csRepoLinks[liIndex]
            getResults[csString] = res.body.Result
            liIndex = liIndex + 1
          }
          cb()
        }
        const afGetTasks = []
        afGetTasks.push(loRepository.get_afTask2Get(coAbstractLib.TYPES.OBJECTS))
        afGetTasks.push(lfcb)
        afGetTasks.push(loRepository.get_afTask2Get(coAbstractLib.TYPES.PROVIDERS))
        afGetTasks.push(lfcb)
        afGetTasks.push(loRepository.get_afTask2Get(coAbstractLib.TYPES.OBJECT_TYPES))
        logger.trace('Create deployment container ...')
        afGetTasks.push(lfcb)
        afGetTasks.push(function () {
          logger.log('Create deployment container ... ok')
          // creating an object of container.js and passing results(only resulted array of objects) of previous GET call on url/repo/objects
          try {
            loContainer = new coContainerLib.Container(credentials, options, iaaFiles, getResults)
          } catch (e) {
            cb(e)
            return
          }

          logger.trace('Prepare step ...')
          const loResultPrepare = loContainer.loResultPrepare
          if (loResultPrepare[coAbstractLib.csKey_nStatus] === coAbstractLib.cnSuccess) {
            logger.log('Prepare step ... ok')

            logger.trace('Create tasks step ...')
            const losExt2oValueFromLoResultPrepare = loResultPrepare[coAbstractLib.csKey_o_sExt2oValue]
            const loResultCreateTasks = loContainer.createTasks(losExt2oValueFromLoResultPrepare)
            if (loResultCreateTasks[coAbstractLib.csKey_nStatus] === coAbstractLib.cnSuccess) {
              logger.log('Create tasks step ... ok')

              logger.trace('Run tasks step ...')
              const losExt2oValueFromLoResultCreateTasks = loResultCreateTasks[coAbstractLib.csKey_o_sExt2oValue]
              loContainer.runTasks(losExt2oValueFromLoResultCreateTasks, function (ioError) {
                if (typeof ioError === 'undefined' || ioError === null) {
                  logger.log('Run tasks step ... ok')
                }
                cb(ioError)
              })
            } else {
              cb(new Error('Failed, because the the create tasks step was not successful, but ' + JSON.stringify(loResultPrepare)))
            }
          } else {
            cb(new Error('Failed, because the the prepare step was not successful, but ' + JSON.stringify(loResultPrepare)))
          }
        })
        coAsyncLib.waterfall(afGetTasks, function (err) {
          if (err) {
            cb(err)
          }
        })
      } catch (e) {
        cb(e)
      }
    }
  }

  return {

    designToRuntime: designToRuntime

  }
}
