'use strict'

const fs = require('fs')
const util = require('util')

let cbIsTraceEnabled = process.env.TRACE

exports.fnDeployServiceSuccess = function (isContainerName) {
  const deployId = process.env.DEPLOY_ID || 'Deployment ID: none'
  console.log('The deployment of DWF content to \'%s\' container finished successfully [%s].', isContainerName, deployId)
}

exports.fnDeployServiceError = function (isContainerName) {
  const deployId = process.env.DEPLOY_ID || 'Deployment ID: none'
  console.error('The deployment of DWF content to \'%s\' container failed [%s].', isContainerName, deployId)
}

function handleSensitiveData (str) {
  str = str.replace(/USER.*/ig, 'u[...]')
  str = str.replace(/PASS.*/ig, 'p[...]')
  return str
}

function writeToStdOut () {
  let str = util.format.apply(null, arguments) + '\n'
  str = handleSensitiveData(str)
  process.stdout.write(str)
}

function writeToStdErr () {
  let str = util.format.apply(null, arguments) + '\n'
  str = handleSensitiveData(str)
  process.stderr.write(str)
}

exports.log = writeToStdOut

exports.warn = writeToStdOut

exports.error = writeToStdErr

exports.setTrace = function (ibIsTraceEnabled) {
  cbIsTraceEnabled = ibIsTraceEnabled
}

exports.trace = function () {
  if (cbIsTraceEnabled) {
    writeToStdOut.apply(this, arguments)
  }
}

const timers = {}

function timerInit (timer) {
  timers[timer] = process.hrtime()
}

function timerDelta (timer) {
  const then = timers[timer]
  const diff = process.hrtime(then)
  if (then || diff) {
    return '(' + (diff[0]) + 's ' + (diff[1] / 1000000).toFixed(0) + 'ms)'
  } else {
    return ''
  }
}

exports.timerInit = timerInit

exports.timerDelta = timerDelta

function logTimerInit (timer) {
  const args = Array.prototype.slice.call(arguments, 1)
  timerInit(timer)
  writeToStdOut.apply(null, args)
}

function logTimerDelta (timer) {
  const args = Array.prototype.slice.call(arguments, 1)
  const delta = timerDelta(timer)
  args.push(delta)
  writeToStdOut.apply(null, args)
}

exports.logTimerInit = logTimerInit

exports.logTimerDelta = logTimerDelta

exports.logfnTimerInit = function () {
  const args = arguments
  return function (cb) {
    logTimerInit.apply(undefined, args)
    cb()
  }
}

exports.logfnTimerDelta = function () {
  const args = arguments
  return function (cb) {
    logTimerDelta.apply(undefined, args)
    cb()
  }
}

let verbose = true

exports.setVerbose = function (value) {
  verbose = value
}

exports.logVerbose = function () {
  if (verbose) {
    writeToStdOut.apply(null, arguments)
  }
}

let logFile

exports.setLogFile = function (filename) {
  logFile = filename
}

let logFileMessageId = 0

exports.logToFile = function (message) {
  if (logFile) {
    ++logFileMessageId
    message.id = logFileMessageId
    fs.appendFileSync(logFile, JSON.stringify(message) + '\n')
  }
}
